/* stuff for testing libraries via access with menu commands
	93/12/30 aih added icon list test, tear off menu test
	93/12/17 aih created */
	
#include "EventLib.h"
#include "FileDialogLib.h"
#include "FloatMenuLib.h"
#include "MacLib.h"
#include "MemoryLib.h"
#include "NotifyLib.h"
#include "IconListLib.h"
#include "ResourceConstantsLib.h"
#include "WindowLib.h"

#include "ConcatenateTextFolders.h"
#include "WordsInFile.h"

void TestEventTableRegister(void);

/* menu resources */
enum {
	mTest = 1024,
	mTestIconList = 250,
	mTestStandardFile = 251,
	mTestLast
};

/* menu commands */
enum {
	cmdTest = CMD_LAST,
	cmdTestWords,
	cmdTestCatFolders,
	cmdTestSF,
	cmdTestSFMultiple,
	cmdTestSFFolder,
	cmdTestSFGet,
	cmdTestSFPut,
	cmdTestIcon,
	cmdTestIconICON,
	cmdTestIconICN,
	cmdTestIconSICN,
	cmdTestLast
};

static MenuPickType gTestMenus[16] = {
	{ mTest, 0, cmdTest, NULL },
	{ mTest, 4, cmdTestWords, NULL },
	{ mTest, 5, cmdTestCatFolders, NULL },
	{ mTestIconList, 0, cmdTestIcon, NULL },
	{ mTestIconList, 1, cmdTestIconICON, NULL },
	{ mTestIconList, 2, cmdTestIconICN, NULL },
	{ mTestIconList, 3, cmdTestIconSICN, NULL },
	{ mTestStandardFile, 0, cmdTestSF, NULL },
	{ mTestStandardFile, 1, cmdTestSFMultiple, NULL },
	{ mTestStandardFile, 2, cmdTestSFFolder, NULL },
	{ mTestStandardFile, 3, cmdTestSFGet, NULL },
	{ mTestStandardFile, 4, cmdTestSFPut, NULL },
	{ 0, 0, CMD_NONE, NULL },
};

static Boolean TestMenu(const MenuPickType *pick)
{
	FileType file, *fp = &file;
	Boolean handled = false;	/* true if command was handled */
	Boolean executable = true;	/* true if command can be executed */
	OSType textType = 'TEXT';	/* for selecting text files */
	
	switch (pick->cmd) {
	case cmdTestSFMultiple:
	case cmdTestSFFolder:
	case cmdTestCatFolders:
		if (! MacHasStandardFile()) {
			NotifyStop(RLS_ALERT_SYSTEM_FEATURE, "7.0");
			executable = false;
		}
		break;
	}
	if (executable) {
		handled = true;
		switch (pick->cmd) {
		case cmdTestSFMultiple:
			HandleEnd(FileSelectMultiple());
			break;
		case cmdTestSFFolder:
			FileOrFolderSelect(fp, 0, NULL);
			break;
		case cmdTestSFGet:
			FileSFButtonTitle("Get text");
			FileSFGet(fp, 1, &textType);
			break;
		case cmdTestSFPut:
			FileSFButtonTitle("Put");
			FileSFPut(fp, "The prompt string", "The file name"); 
			break;
		case cmdTestIconICON:
			(void) IconListBegin(ICON_LIST_ICON, RLW_TOOL_BAR);
			break;
		case cmdTestIconICN:
			(void) IconListBegin(ICON_LIST_ICON_LIST, RLW_TOOL_BAR);
			break;
		case cmdTestIconSICN:
			(void) IconListBegin(ICON_LIST_SMALL_ICON, RLW_TOOL_BAR);
			break;
		case cmdTestWords:
			WordsInFile(NULL, NULL);
			break;
		case cmdTestCatFolders:
			ConcatenateTextFolders(NULL, 0, NULL);
			break;
		default:
			handled = false;
			break;
		}
	}
	return(handled);
}

static void TestAdjustMenu(void)
{
	if (! WinModalHasFocus()) {
		MenuCmdEnable(cmdTest);
		MenuCmdEnable(cmdTestSF);
		MenuCmdEnable(cmdTestIcon);
		MenuCmdEnable(cmdTestSFMultiple);
		MenuCmdEnable(cmdTestSFFolder);
		MenuCmdEnable(cmdTestSFGet);
		MenuCmdEnable(cmdTestSFPut);
		MenuCmdEnable(cmdTestIconICON);
		MenuCmdEnable(cmdTestIconICN);
		MenuCmdEnable(cmdTestIconSICN);
		MenuCmdEnable(cmdTestWords);
		MenuCmdEnable(cmdTestCatFolders);
	}
}

static Boolean menu(const MenuPickType *pick)
{
	return TestMenu(pick);
}

static void adjustmenu(void)
{
	TestAdjustMenu();
}

void TestEventTableRegister(void)
{
	static EventTableType table;
	
	table.objectType.menu = menu;
	table.objectType.adjustmenu = adjustmenu;
	EventTableRegister(&table);
	MenuAppend(mTest);
	MenuTableAppend(gTestMenus);
	(void) FloatMenuBegin(mTest);
	(void) FloatMenuBegin(mTestIconList);
	(void) FloatMenuBegin(mTestStandardFile);
}
